package proc;

import java.util.ArrayList;

public class ProcParallel extends Proc {
	Proc left;
	Proc right;

	public ProcParallel(Proc left, Proc right) {
		this.type = PARALLEL;
		this.left = left;
		this.right = right;
	}

	public String toString() {
		return "(" + left + " | " + right + ")";
	}

	public ArrayList<Transition> transitions() {
		if(trans != null) return trans;
		
		ArrayList<Transition> ltrans = left.transitions();
		ArrayList<Transition> rtrans = right.transitions();
		trans = new ArrayList<Transition>();
		
		// add transitions of the form P'|Q where P->P'
		for(Transition lt : ltrans) {
			Transition t = lt.clone();
			for(ProbOfProc p : t.dist)
				p.process = new ProcParallel(p.process, right);
			trans.add(t);
		}

		// add transitions of the form P|Q' where Q->Q'
		for(Transition rt : rtrans) {
			Transition t = rt.clone();
			for(ProbOfProc p : t.dist)
				p.process = new ProcParallel(left, p.process);
			trans.add(t);
		}
		
		// add transitions of the form P|Q-tau->P'|Q' where P-a->P' and Q-abar->Q'
		for(Transition lt : ltrans)
			for(Transition rt : rtrans)
				if(lt.channel.equals(rt.channel) && lt.output ^ rt.output) {
					Proc p1 = lt.dist.get(0).process;
					Proc p2 = rt.dist.get(0).process;
					Proc proc = new ProcParallel(p1, p2);
					trans.add(Transition.dirac(lt.label+","+rt.label, "tau", false, proc));
				}

		return trans;
	}
}

